﻿#include <QApplication>
#include <QMainWindow>
#include <QDebug>
#include <QWidget>
#include <QAbstractNativeEventFilter>

#include <qt_windows.h>

static bool optHandleEvent = false;

class Widget : public QWidget {
public:
    bool nativeEvent(const QByteArray &eventType, void *message, long *result) Q_DECL_OVERRIDE;
};

bool Widget::nativeEvent(const QByteArray &, void *messageIn, long *)
{
    MSG* msg = static_cast<MSG*>(messageIn);
    if (msg->message != WM_TIMER && msg->message != WM_USER + 1)
        qDebug().noquote().nospace() << __FUNCTION__ << ' ' << *msg;
    return false;
}

class NativeEventFilter : public QAbstractNativeEventFilter
{
public:
    bool nativeEventFilter(const QByteArray &eventType, void *message, long *result) Q_DECL_OVERRIDE;
};

bool NativeEventFilter::nativeEventFilter(const QByteArray &,
                                          void *messageIn,
                                          long *result)
{
    MSG* msg = static_cast<MSG*>(messageIn);
    if (msg->message != WM_TIMER && msg->message != WM_USER + 1)
        qDebug().noquote().nospace() << __FUNCTION__ << ' ' << *msg;
    if (!optHandleEvent)
        return false;

    if (msg->message == WM_NCHITTEST)
        qDebug() << "Message WM_NCHITTEST";

    QWidget* widget = QWidget::find(reinterpret_cast<WId>(msg->hwnd));
    if (!widget)
        return false;

    switch (msg->message) {
    case WM_NCCALCSIZE:
        *result = 0;
        return true;
    case WM_NCHITTEST:
        *result = HTCAPTION;
        return true;
    default:
        break;
    }

    return false;
}

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    QStringList args = a.arguments();
    args.pop_front();
    qDebug() << QT_VERSION_STR << args;
    optHandleEvent = args.contains("-h");

    Widget w;
    w.setWindowTitle(QT_VERSION_STR);
    if (!args.contains("-f"))
        w.setWindowFlags(Qt::FramelessWindowHint | Qt::Window);
    w.resize(200,200);
    w.show();

    NativeEventFilter *nf = new NativeEventFilter;

    qApp->installNativeEventFilter(nf);

    return a.exec();
}
